﻿// <copyright file="DnsRecordRepository.cs" company="Lakstins Family, LLC">
// Copyright (c) Brian A. Lakstins (http://www.lakstins.com/brian/)
// </copyright>

#region License
// <license>
// This software is provided 'as-is', without any express or implied warranty. In no 
// event will the author be held liable for any damages arising from the use of this 
// software.
//  
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject to the 
// following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not claim that 
// you wrote the original software. If you use this software in a product, an 
// acknowledgment (see the following) in the product documentation is required.
// 
// Portions Copyright (c) Brian A. Lakstins (http://www.lakstins.com/brian/)
// 
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
// 
// 3. This notice may not be removed or altered from any source distribution.
// </license>
#endregion

#region Change Log
// <changelog>
// <change date="6/14/2010" author="Brian A. Lakstins" description="Initial Release">
// </changelog>
#endregion

namespace BalCoding.DurableDNSApi.DataAccess
{
	using System;
	using System.Collections.Generic;
	using System.Data;

	/// <summary>
	/// Repository used to manage Dns Record data using the DuralbleDNS Api
	/// </summary>
	public static class BalDnsRecordRepository
	{
		/// <summary>
		/// Gets a template to use for insert and update.
		/// </summary>
		/// <returns>A DataTable with columns that match record data</returns>
		public static DataTable GetBlankTable()
		{
			DataTable loTable = new DataTable();
			loTable.Columns.Add(new DataColumn(RecordMap.Id, typeof(int)));
			loTable.Columns.Add(new DataColumn(RecordMap.Name, typeof(string)));
			loTable.Columns.Add(new DataColumn(RecordMap.Data, typeof(string)));
			loTable.Columns.Add(new DataColumn(RecordMap.DDNS, typeof(string)));
			loTable.Columns.Add(new DataColumn(RecordMap.Type, typeof(string)));
			loTable.Columns.Add(new DataColumn(RecordMap.Aux, typeof(int)));
			loTable.Columns.Add(new DataColumn(RecordMap.TTL, typeof(int)));
			return loTable;
		}

		/// <summary>
		/// Creates a new DNS record within an already existing DNS zone.
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="loTable">Table with a row for each record to be added</param>
		/// <returns>List of IDs that were added</returns>
		public static string[] Insert(string lsUser, string lsKey, string lsZone, DataTable loTable)
		{
			List<string> loList = new List<string>();
			foreach (DataRow loRow in loTable.Rows)
			{
				string lsName = loRow[RecordMap.Name].ToString();
				string lsType = loRow[RecordMap.Type].ToString(); 
				string lsData = loRow[RecordMap.Data].ToString();
				string lsDDNS = loRow[RecordMap.DDNS].ToString(); 
				int lnAux = (int)loRow[RecordMap.Aux];
				int lnTTL = (int)loRow[RecordMap.TTL];

				loList.Add(new DurableDns.Record.Create.createRecordwsdl().createRecord(lsUser, lsKey, lsZone, lsName, lsType, lsData, lnAux, lnTTL, lsDDNS));
			}

			return loList.ToArray();
		}

		/// <summary>
		/// Updates a DNS record
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="lnRecordId">Unique numeric record ID</param>
		/// <param name="loTable">Table with rows to be updated</param>
		public static void Update(string lsUser, string lsKey, string lsZone, int lnRecordId, DataTable loTable)
		{
			foreach (DataRow loRow in loTable.Rows)
			{
				string lsName = loRow[RecordMap.Name].ToString();
				string lsType = loRow[RecordMap.Type].ToString();
				string lsData = loRow[RecordMap.Data].ToString();
				string lsDDNS = loRow[RecordMap.DDNS].ToString();
				int lnAux = (int)loRow[RecordMap.Aux];
				int lnTTL = (int)loRow[RecordMap.TTL];

				try
				{
					string lsSuccess = new DurableDns.Record.Update.updateRecordwsdl().updateRecord(lsUser, lsKey, lsZone, lnRecordId, lsName, lnAux, lsData, lnTTL, lsDDNS);
					if (!lsSuccess.Equals("success", StringComparison.InvariantCultureIgnoreCase))
					{
						throw new Exception(lsSuccess);
					}
				}
				catch (Exception loE)
				{
					throw new Exception("Error updating [" + lsZone + "][" + lnRecordId.ToString() + "][" + lsName + "][" + lsType + "][" + lsData + "]", loE);
				}
			}
		}

		/// <summary>
		/// Deletes a DNS record from a zone
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="lnRecordId">Unique numeric record ID</param>
		public static void Delete(string lsUser, string lsKey, string lsZone, int lnRecordId)
		{
			string lsSuccess = new DurableDns.Record.Delete.deleteRecordwsdl().deleteRecord(lsUser, lsKey, lsZone, lnRecordId);
			if (!lsSuccess.Equals("success", StringComparison.InvariantCultureIgnoreCase))
			{
				throw new Exception(lsSuccess);
			}
		}

		/// <summary>
		/// Get details of a DNS record by unique numeric ID
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="lnRecordId">Unique numeric record ID</param>
		/// <returns>Datatable with one row for the record</returns>
		public static DataTable Select(string lsUser, string lsKey, string lsZone, int lnRecordId)
		{
			string lsName = string.Empty;
			string lsType = string.Empty;
			string lsData = string.Empty;
			int lnAux = 0;
			int lnTTL = 0;
			int lnId = new DurableDns.Record.Get.getRecordwsdl().getRecord(lsUser, lsKey, lsZone, lnRecordId, out lsName, out lsType, out lsData, out lnAux, out lnTTL);

			DataTable loTable = GetBlankTable();
			if (lnId.Equals(lnRecordId))
			{
				DataRow loRow = loTable.NewRow();
				loRow[RecordMap.Id] = lnId;
				loRow[RecordMap.Name] = lsName;
				loRow[RecordMap.Type] = lsType;
				loRow[RecordMap.Data] = lsData;
				loRow[RecordMap.Aux] = lnAux;
				loRow[RecordMap.TTL] = lnTTL;
				loTable.Rows.Add(loRow);
			}

			return loTable;
		}

		/// <summary>
		/// Get the list of records for a specified DNS zone.  Does not include Aux or TTL data for each record.
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <returns>Datatable with a row for each record in the zone</returns>
		public static DataTable Select(string lsUser, string lsKey, string lsZone)
		{
			DurableDns.Record.List.Record[] laRecord = new DurableDns.Record.List.listRecordswsdl().listRecords(lsUser, lsKey, lsZone);
			DataTable loTable = GetBlankTable();
			foreach (DurableDns.Record.List.Record loRecord in laRecord)
			{
				DataRow loRow = loTable.NewRow();
				loRow[RecordMap.Id] = loRecord.id;
				loRow[RecordMap.Name] = loRecord.name;
				loRow[RecordMap.Type] = loRecord.type;
				loRow[RecordMap.Data] = loRecord.data;
				loTable.Rows.Add(loRow);
			}

			return loTable;
		}

		/// <summary>
		/// Gets a list of internal record Ids for DNS Records in a zone
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <returns>Array of record Ids</returns>
		public static int[] SelectIds(string lsUser, string lsKey, string lsZone)
		{
			try
			{
				DurableDns.Record.List.Record[] laRecord = new DurableDns.Record.List.listRecordswsdl().listRecords(lsUser, lsKey, lsZone);
				List<int> loList = new List<int>();
				foreach (DurableDns.Record.List.Record loRecord in laRecord)
				{
					loList.Add(loRecord.id);
				}

				return loList.ToArray();
			}
			catch (Exception loE)
			{
				throw new Exception("Error getting Ids for [" + lsZone + "]", loE);
			}
		}

		/// <summary>
		/// Tracks the data table column mapping
		/// </summary>
		public class RecordMap
		{
			/// <summary>
			/// Internal DurableDns Id
			/// </summary>
			public const string Id = "id";

			/// <summary>
			/// The record name is generally a host name.
			/// </summary>
			public const string Name = "name";

			/// <summary>
			/// DNS record type
			/// A – Address Record – Most commonly used to map hostnames to an IPv4 address of the host.
			/// AAAA – IPv6 Address Record – Most commonly used to maps hostnames to an IPv6 address of the host.
			/// CNAME – Canonical Name Record – Alias of one name to another.
			/// MX – Mail Exchange Record – Maps a domain name to a list of mail exchange servers (SMTP) for that domain.
			/// NS – Name Server Record – Delegates a DNS zone to use the given authoritative name servers.
			/// PTR – Pointer Record – Pointer to a canonical name. Unlike a CNAME, DNS processing does not proceed, just the name is returned. PTR is most commonly used when implementing reverse DNS lookups.
			/// SRV – Service Locator – Generalized service location record, used for newer protocols instead of creating protocol-specific records such as MX.
			/// TXT – Text Record – Originally for arbitrary human-readable text in a DNS record. Since the early 1990s, however, this record more often carries machine-readable data, such as Sender Policy Framework, DomainKeys, DNS-SD, etc.
			/// </summary>
			public const string Type = "type";

			/// <summary>
			/// The record data is generally an IP address (for an A record) or a hostname (for a CNAME, MX, or NS record).
			/// </summary>
			public const string Data = "data";

			/// <summary>
			/// Allow Dynamic DNS API Updates (Y or N)
			/// </summary>
			public const string DDNS = "ddns";

			/// <summary>
			/// Priority for MX record types
			/// </summary>
			public const string Aux = "aux";

			/// <summary>
			///  The TTL is the time, in seconds, that resolving DNS servers are allowed to cache the record.
			/// </summary>
			public const string TTL = "ttl";
		}
	}
}
