﻿// <copyright file="DnsZoneRepository.cs" company="Lakstins Family, LLC">
// Copyright (c) Brian A. Lakstins (http://www.lakstins.com/brian/)
// </copyright>

#region License
// <license>
// This software is provided 'as-is', without any express or implied warranty. In no 
// event will the author be held liable for any damages arising from the use of this 
// software.
//  
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject to the 
// following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not claim that 
// you wrote the original software. If you use this software in a product, an 
// acknowledgment (see the following) in the product documentation is required.
// 
// Portions Copyright (c) Brian A. Lakstins (http://www.lakstins.com/brian/)
// 
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
// 
// 3. This notice may not be removed or altered from any source distribution.
// </license>
#endregion

#region Change Log
// <changelog>
// <change date="6/14/2010" author="Brian A. Lakstins" description="Initial Release">
// </changelog>
#endregion

namespace BalCoding.DurableDNSApi.DataAccess
{
	using System;
	using System.Collections.Generic;
	using System.Data;

	/// <summary>
	/// Repository used to manage Dns Zone data using the DuralbleDNS Api
	/// </summary>
	public static class BalDnsZoneRepository
	{
		/// <summary>
		/// Gets a template to use for insert and update.
		/// </summary>
		/// <returns>A DataTable with columns that match zone data</returns>
		public static DataTable GetBlankTable()
		{
			DataTable loTable = new DataTable();
			loTable.Columns.Add(new DataColumn(ZoneMap.Name, typeof(string)));
			loTable.Columns.Add(new DataColumn(ZoneMap.NameServer, typeof(string)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Email, typeof(string)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Xfer, typeof(string)));
			loTable.Columns.Add(new DataColumn(ZoneMap.UpdateACL, typeof(string)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Serial, typeof(int)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Refresh, typeof(int)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Retry, typeof(int)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Expire, typeof(int)));
			loTable.Columns.Add(new DataColumn(ZoneMap.Minimum, typeof(int)));
			loTable.Columns.Add(new DataColumn(ZoneMap.TTL, typeof(int)));
			return loTable;
		}

		/// <summary>
		/// Creates a new DNS zone.
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="loTable">Table with rows of data to be updated</param>
		/// <returns>Array of zone Ids that were created</returns>
		public static string[] Insert(string lsUser, string lsKey, string lsZone, DataTable loTable)
		{
			List<string> loList = new List<string>();
			foreach (DataRow loRow in loTable.Rows)
			{
				string lsNameServer = loRow[ZoneMap.NameServer].ToString();
				string lsEmail = loRow[ZoneMap.Email].ToString();
				int lnSerial = (int)loRow[ZoneMap.Serial];
				int lnRefresh = (int)loRow[ZoneMap.Refresh];
				int lnRetry = (int)loRow[ZoneMap.Retry];
				int lnExpire = (int)loRow[ZoneMap.Expire];
				int lnMinimum = (int)loRow[ZoneMap.Minimum];
				int lnTTL = (int)loRow[ZoneMap.TTL];
				string lsXfer = loRow[ZoneMap.Xfer].ToString();
				string lsUpdateACL = loRow[ZoneMap.UpdateACL].ToString();

				loList.Add(new DurableDns.Zone.Create.createZonewsdl().createZone(lsUser, lsKey, lsZone, lsNameServer, lsEmail, lnRefresh, lnRetry, lnExpire, lnMinimum, lnTTL, lsXfer, lsUpdateACL));
			}

			return loList.ToArray();
		}

		/// <summary>
		/// Update DNS Zones
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <param name="loTable">Table with rows of data to be updated</param>
		public static void Update(string lsUser, string lsKey, string lsZone, DataTable loTable)
		{
			foreach (DataRow loRow in loTable.Rows)
			{
				string lsNameServer = loRow[ZoneMap.NameServer].ToString();
				string lsEmail = loRow[ZoneMap.Email].ToString();
				int lnSerial = (int)loRow[ZoneMap.Serial];
				int lnRefresh = (int)loRow[ZoneMap.Refresh];
				int lnRetry = (int)loRow[ZoneMap.Retry];
				int lnExpire = (int)loRow[ZoneMap.Expire];
				int lnMinimum = (int)loRow[ZoneMap.Minimum];
				int lnTTL = (int)loRow[ZoneMap.TTL];
				string lsXfer = loRow[ZoneMap.Xfer].ToString();
				string lsUpdateACL = loRow[ZoneMap.UpdateACL].ToString();

				string lsSuccess = new DurableDns.Zone.Update.updateZonewsdl().updateZone(lsUser, lsKey, lsZone, lsNameServer, lsEmail, lnRefresh, lnRetry, lnExpire, lnMinimum, lnTTL, lsXfer, lsUpdateACL);
				if (!lsSuccess.Equals("success", StringComparison.InvariantCultureIgnoreCase))
				{
					throw new Exception(lsSuccess);
				}
			}
		}

		/// <summary>
		/// Deletes a DNS zone.
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		public static void Delete(string lsUser, string lsKey, string lsZone)
		{
			string lsSuccess = new DurableDns.Zone.Delete.deleteZonewsdl().deleteZone(lsUser, lsKey, lsZone);
			if (!lsSuccess.Equals("success", StringComparison.InvariantCultureIgnoreCase))
			{
				throw new Exception(lsSuccess);
			}
		}

		/// <summary>
		/// List all zone names associated with account
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <returns>List of domain zones being managed by this account</returns>
		public static string[] SelectNames(string lsUser, string lsKey)
		{
			DurableDns.Zone.List.Zone[] laZone = new DurableDns.Zone.List.listZoneswsdl().listZones(lsUser, lsKey);

			List<string> loList = new List<string>();
			foreach (DurableDns.Zone.List.Zone loZone in laZone)
			{
				loList.Add(loZone.origin);
			}

			return loList.ToArray();
		}

		/// <summary>
		/// List all zones associated with account
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <returns>DataTable with all records for the specified account</returns>
		public static DataTable Select(string lsUser, string lsKey)
		{
			DurableDns.Zone.List.Zone[] laZone = new DurableDns.Zone.List.listZoneswsdl().listZones(lsUser, lsKey);

			DataTable loTable = GetBlankTable();
			foreach (DurableDns.Zone.List.Zone loZone in laZone)
			{
				string lsNameServer = string.Empty;
				string lsEmail = string.Empty;
				int lnSerial = 0;
				int lnRefresh = 0;
				int lnRetry = 0;
				int lnExpire = 0;
				int lnMinimum = 0;
				int lnTTL = 0;
				string lsXfer = string.Empty;
				string lsUpdateACL = string.Empty;
				string lsSuccess = new DurableDns.Zone.Get.getZonewsdl().getZone(lsUser, lsKey, loZone.origin, out lsNameServer, out lsEmail, out lnSerial, out lnRefresh, out lnRetry, out lnExpire, out lnMinimum, out lnTTL, out lsXfer, out lsUpdateACL);

				if (lsSuccess.Equals(loZone.origin, StringComparison.InvariantCultureIgnoreCase))
				{
					DataRow loRow = loTable.NewRow();
					loRow[ZoneMap.Name] = loZone.origin;
					loRow[ZoneMap.NameServer] = lsNameServer;
					loRow[ZoneMap.Email] = lsEmail;
					loRow[ZoneMap.Serial] = lnSerial;
					loRow[ZoneMap.Refresh] = lnRefresh;
					loRow[ZoneMap.Retry] = lnRetry;
					loRow[ZoneMap.Expire] = lnExpire;
					loRow[ZoneMap.Minimum] = lnMinimum;
					loRow[ZoneMap.TTL] = lnTTL;
					loRow[ZoneMap.Xfer] = lsXfer;
					loRow[ZoneMap.UpdateACL] = lsUpdateACL;
					loTable.Rows.Add(loRow);
				}
			}

			return loTable;
		}

		/// <summary>
		/// Get details of DNS zone
		/// </summary>
		/// <param name="lsUser">API User: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsKey">API Key: Value for authentication – Provided by DurableDNS on the Dashboard</param>
		/// <param name="lsZone">Name of zone, followed by a dot (.). Example: "example.com."</param>
		/// <returns>DataTable with one record for the specified zone</returns>
		public static DataTable Select(string lsUser, string lsKey, string lsZone)
		{
			string lsNameServer = string.Empty;
			string lsEmail = string.Empty;
			int lnSerial = 0;
			int lnRefresh = 0;
			int lnRetry = 0;
			int lnExpire = 0;
			int lnMinimum = 0;
			int lnTTL = 0;
			string lsXfer = string.Empty;
			string lsUpdateACL = string.Empty;

			string lsSuccess = new DurableDns.Zone.Get.getZonewsdl().getZone(lsUser, lsKey, lsZone, out lsNameServer, out lsEmail, out lnSerial, out lnRefresh, out lnRetry, out lnExpire, out lnMinimum, out lnTTL, out lsXfer, out lsUpdateACL);
			
			DataTable loTable = GetBlankTable();
			if (lsSuccess.Equals(lsZone, StringComparison.InvariantCultureIgnoreCase))
			{
				DataRow loRow = loTable.NewRow();
				loRow[ZoneMap.Name] = lsZone;
				loRow[ZoneMap.NameServer] = lsNameServer;
				loRow[ZoneMap.Email] = lsEmail;
				loRow[ZoneMap.Serial] = lnSerial;
				loRow[ZoneMap.Refresh] = lnRefresh;
				loRow[ZoneMap.Retry] = lnRetry;
				loRow[ZoneMap.Expire] = lnExpire;
				loRow[ZoneMap.Minimum] = lnMinimum;
				loRow[ZoneMap.TTL] = lnTTL;
				loRow[ZoneMap.Xfer] = lsXfer;
				loRow[ZoneMap.UpdateACL] = lsUpdateACL;
				loTable.Rows.Add(loRow);
			}

			return loTable;
		}
		
		/// <summary>
		/// Tracks the data table column mapping
		/// </summary>
		public class ZoneMap
		{
			/// <summary>
			/// Name of zone to create, followed by a dot (.). Example: "example.com."
			/// </summary>
			public const string Name = "name";

			/// <summary>
			///  Fully-qualified domain name of primary name server for zone. Example: "ns1.alwaysdnsllc.com."
			///  Primary Nameserver:
			///  Unless DurableDNS will act as secondary DNS to another DNS server you have hosted elsewhere, 
			///  the primary nameserver will always be NS1.ALWAYSDNSLLC.COM. If you are going to private-label 
			///  our nameservers (i.e. use custom hostname for our IP addresses), you can set this to your 
			///  private-label nameserver host. This should end in a dot unless you are not entering a 
			///  fully qualified domain name.
			/// </summary>
			public const string NameServer = "nameserver";

			/// <summary>
			/// Email address of person responsible for the zone. Example: "support.durabledns.com" (Replace the '@' with a '.')
			/// Responsible Person Mailbox: (hostmaster.example.com.)
			///	This is the email address of the person responsible for this DNS zone. It must be in DNS format 
			///	which means you need to replace the '@' with a dot '.'. This field must end in a dot or the zone 
			///	name will be appended to the record.
			/// </summary>
			public const string Email = "email";

			/// <summary>
			/// Current serial number of zone
			/// Serial:
			/// This is the serial number of the zone. There is almost no reason to change this.
			/// As records are updated, it is set to the number of seconds that have elapsed since 1/1/1970.
			/// </summary>
			public const string Serial = "serial";

			/// <summary>
			/// Zone refresh time in seconds
			/// Refresh: (seconds)
			/// This is the amount of time secondary nameservers should wait before refreshing 
			/// their copy of this zone. This only applies if you have non-DurableDNS nameservers 
			/// that get data from DurableDNS nameservers.
			/// </summary>
			public const string Refresh = "refresh";

			/// <summary>
			/// Zone retry time in seconds
			/// Retry: (seconds)
			///	This is the amount of time secondary nameservers should wait before retrying a failed 
			///	zone download. This only applies if you have non-DurableDNS nameservers that get data 
			///	from DurableDNS nameservers.
			/// </summary>
			public const string Retry = "retry";

			/// <summary>
			/// Zone expiration time in seconds
			/// Expire: (seconds)
			///	This is the amount of time secondary nameservers should wait before deleting a zone 
			///	from their records if they have been unable to update from the primary nameserver. 
			///	This only applies if you have non-DurableDNS nameservers that get data from DurableDNS nameservers.
			/// </summary>
			public const string Expire = "expire";

			/// <summary>
			/// Minimum TTL for records within zone, in seconds
			/// Minimum Time-to-live for Records: (seconds)
			///	This is the minimum allowed TTL value for records in this zone. If a record has a TTL lower 
			///	than this value, the DNS servers will respond with this value.
			/// </summary>
			public const string Minimum = "minimum";

			/// <summary>
			/// TTL for zone (SOA record), in seconds
			/// Zone Time-to-live: (seconds)
			///	This is the amount of time that resolving DNS servers and DNS clients are allowed to cache this zone.
			/// </summary>
			public const string TTL = "ttl";

			/// <summary>
			/// AXFR allow list, comma separated IP addresses in CIDR format. Example: “127.0.0.1/32,127.0.0.100/32”
			/// AXFR Zone Transfer Allow List: (comma separated, CIDR format)
			///	This is a list of IP addresses or IP address ranges that are allowed to download zone data via AXFR. 
			///	This is not commonly used.
			/// </summary>
			public const string Xfer = "xfer";

			/// <summary>
			/// DDNS update ACL, comma separated IP addresses in CIDR format. Example: “127.0.0.1/32,127.0.0.100/32”
			/// DDNS Update Allow List: (comma separated, CIDR format)
			///	This is a list of IP address or IP address ranges that are allowed to update records via DDNS. 
			///	This is not commonly used.
			/// </summary>
			public const string UpdateACL = "updateacl";
		}
	}
}
